﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarDataModels.Models;

namespace SushiBarView
{
    public partial class FormSushi : Form
    {
        private readonly ILogger _logger;
        private readonly ISushiLogic _logic;
        private int? _id;
        private Dictionary<int, (IIngredientModel, int)> _sushiIngredients;
        public int Id { set { _id = value; } }
        public FormSushi(ILogger<FormSushi> logger, ISushiLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
            _sushiIngredients = new Dictionary<int, (IIngredientModel, int)>();
        }
        private void FormSushi_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка суши");
                try
                {
                    var view = _logic.ReadElement(new SushiSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxName.Text = view.SushiName;
                        textBoxPrice.Text = view.Price.ToString();
                        _sushiIngredients = view.SushiIngredients ?? new Dictionary<int, (IIngredientModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки суши");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка ингредиентов суши");
            try
            {
                if (_sushiIngredients != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var sc in _sushiIngredients)
                    {
                        dataGridView.Rows.Add(new object[] { sc.Key, sc.Value.Item1.IngredientName, sc.Value.Item2 });
                    }
                    textBoxPrice.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки ингредиентов суши");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormSushiIngredients));
            if (service is FormSushiIngredients form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.IngredientModel == null)
                    {
                        return;
                    }
                    _logger.LogInformation("Добавление нового ингредиента: { IngredientName} - { Count}", form.IngredientModel.IngredientName, form.Count);
                    if (_sushiIngredients.ContainsKey(form.Id))
                    {
                        _sushiIngredients[form.Id] = (form.IngredientModel, form.Count);
                    }
                    else
                    {
                        _sushiIngredients.Add(form.Id, (form.IngredientModel, form.Count));
                    }
                    LoadData();
                }
            }
        }
        private void ButtonUpd_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormSushiIngredients));
                if (service is FormSushiIngredients form)
                {
                    int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value);
                    form.Id = id;
                    form.Count = _sushiIngredients[id].Item2;
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.IngredientModel == null)
                        {
                            return;
                        }
                        _logger.LogInformation("Изменение ингредиента: { IngredientName} - { Count} ", form.IngredientModel.IngredientName, form.Count);
                        _sushiIngredients[form.Id] = (form.IngredientModel, form.Count);
                        LoadData();
                    }
                }
            }
        }
        private void ButtonDel_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление ингредиента: { IngredientName} - { Count} ", 
                            dataGridView.SelectedRows[0].Cells[1].Value); _sushiIngredients?.Remove(Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    LoadData();
                }
            }
        }
        private void ButtonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxPrice.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (_sushiIngredients == null || _sushiIngredients.Count == 0)
            {
                MessageBox.Show("Заполните ингредиенты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение суши");
            try
            {
                var model = new SushiBindingModel
                {
                    Id = _id ?? 0,
                    SushiName = textBoxName.Text,
                    Price = Convert.ToDouble(textBoxPrice.Text),
                    SushiIngredients = _sushiIngredients
                };
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения суши");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
        private double CalcPrice()
        {
            double price = 0;
            foreach (var elem in _sushiIngredients)
            {
                price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
            }
            return Math.Round(price * 1.1, 2);
        }
    }
}
